package
{
	import away3d.cameras.lenses.PerspectiveLens;
	import away3d.containers.ObjectContainer3D;
	import away3d.containers.View3D;
	import away3d.entities.Mesh;
	import away3d.events.LoaderEvent;
	import away3d.loaders.Loader3D;
	import away3d.loaders.parsers.Max3DSParser;
	import away3d.materials.ColorMaterial;
	import away3d.materials.TextureMaterial;
	import away3d.textures.BitmapTexture;
	import flash.display.Bitmap;
	import flash.display.MovieClip;
	import flash.display.Shape;
	import flash.display.SimpleButton;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.IOErrorEvent;
	import flash.events.MouseEvent;
	import flash.events.SecurityErrorEvent;
	import flash.geom.Vector3D;
	import flash.net.FileFilter;
	import flash.net.FileReference;
	import flash.net.URLRequest;
	import flash.system.Security;
	import flash.text.TextField;
	
	// 3DS Model Viewer Tutorial
	// by Vu Hoang Minh - www.3dgameflash.com
	// Created for active.tutsplus.com
	
	public class Main extends Sprite
	{		
		private var file:FileReference;
		private var view3d:View3D;
		private var loader:Loader3D;
		private var labelDebug:TextField;
		private var object3d:Mesh;
		
		public function Main():void
		{
			if (stage)
				init();
			else
				addEventListener(Event.ADDED_TO_STAGE, init);
		}
		
		private function init(e:Event = null):void
		{
			//allow all domain for load local file
			Security.allowDomain("*");
			
			removeEventListener(Event.ADDED_TO_STAGE, init);
			//3d engine
			view3d = new View3D();
			view3d.camera.lens = new PerspectiveLens();
			view3d.camera.z = 100;
			addChild(view3d);
			
			//init 3d loader
			initLoader();
			
			//mcBrowse
			var mcBrowse:MovieClip = new MovieClip();
			mcBrowse.graphics.beginFill(0xeeeeee);
			mcBrowse.graphics.drawRoundRect(1, 2, 100, 25, 7, 7);
			mcBrowse.graphics.endFill();
			var labelBrowse:TextField = new TextField();
			labelBrowse.text = "Browse";
			mcBrowse.addChild(labelBrowse);
			mcBrowse.mouseChildren = false;
			mcBrowse.buttonMode = true;
			labelBrowse.x = 25;
			mcBrowse.addEventListener(MouseEvent.CLICK, onClick_mcBrowse);
			addChild(mcBrowse);
			
			//debug label
			labelDebug = new TextField();
			labelDebug.text = "...";
			labelDebug.textColor = 0xff0000;
			labelDebug.selectable = false;
			labelDebug.y = mcBrowse.height / 2;
			labelDebug.width = 600;
			addChild(labelDebug);
			
			//file
			file = new FileReference();
		}
		
		private function initLoader():void
		{
			//clear
			removeEventListener(Event.ENTER_FRAME, onEnterFrame);			
			
			while (view3d.scene.numChildren > 0)
			{
				view3d.scene.removeChild(view3d.scene.getChildAt(0));
			}
			
			//init new Loader
			Loader3D.enableParser(Max3DSParser);
			loader = new Loader3D();
			loader.addEventListener(LoaderEvent.RESOURCE_COMPLETE, onComplete_loader);
			loader.addEventListener(LoaderEvent.LOAD_ERROR, onError_loader);
			view3d.scene.addChild(loader);
			view3d.camera.lookAt(loader.position);
		}
	
		private function onClick_mcBrowse(e:MouseEvent):void
		{
			file.browse([new FileFilter("3DS", "*.3ds")]);
			file.addEventListener(Event.SELECT, onFileSelected);
			file.addEventListener(SecurityErrorEvent.SECURITY_ERROR, onSecurityError);
			file.addEventListener(IOErrorEvent.IO_ERROR, onIOError);
		}
		
		private function onSecurityError(e:Event):void 
		{
			labelDebug.text += ".Security Error!";
		}
		
		private function onIOError(e:IOErrorEvent):void 
		{
			labelDebug.text += ".File not found Error!";
		}
		
		private function onFileSelected(e:Event):void
		{
			labelDebug.text = "File :" + file.name;
			file.removeEventListener(Event.SELECT, onFileSelected);
			file.addEventListener(Event.COMPLETE, onFileLoaded);
			file.load();
		}
		
		private function onFileLoaded(e:Event):void
		{
			file.removeEventListener(Event.COMPLETE, onFileLoaded);
			initLoader();
			loader.loadData(e.target.data);
		}
		
		private function onError_loader(e:LoaderEvent):void
		{
			trace("Error loading File...");
			labelDebug.text += " .Loading Error";
		}
		
		private function onComplete_loader(e:LoaderEvent):void
		{			
			trace("3D File loaded");
			labelDebug.text += " .Complete.Rendering...";
			loader.removeEventListener(LoaderEvent.RESOURCE_COMPLETE, onComplete_loader);
			loader.removeEventListener(LoaderEvent.LOAD_ERROR, onError_loader);
			object3d = Mesh(loader.getChildAt(0));
			view3d.scene.addChild(object3d);
			loader.dispose();
			loader = null;
			
			addEventListener(Event.ENTER_FRAME, onEnterFrame);
		}
		
		private function onEnterFrame(e:Event):void
		{
			object3d.yaw(1);
			view3d.render();
		}
	}

}